@@ -1,11 +1,14 @@
 #!/usr/bin/env python
 ###############################################################################
+#  $Id: gdal_retile.py 28389 2015-01-30 19:26:09Z rouault $
 #
 # Purpose:  Module for retiling (merging) tiles and building tiled pyramids
 # Author:   Christian Meuller, christian.mueller@nvoe.at
+# UseDirForEachRow support by Chris Giesey & Elijah Robison
 #
 ###############################################################################
 # Copyright (c) 2007, Christian Mueller
+# Copyright (c) 2009-2012, Even Rouault <even dot rouault at mines-paris dot org>
 #
 # Permission is hereby granted, free of charge, to any person obtaining a
 # copy of this software and associated documentation files (the "Software"),
@@ -31,19 +34,22 @@ try:
     from osgeo import gdal
     from osgeo import ogr
     from osgeo import osr
-    from osgeo.gdalconst import *
 except:
     import gdal
     import ogr
     import osr
-    from gdalconst import *
 
 import sys
 import os
 import math
 
+try:
+    progress = gdal.TermProgress_nocb
+except:
+    progress = gdal.TermProgress
+
 class AffineTransformDecorator:
-    """ A class providing some usefull methods for affine Transformations """
+    """ A class providing some useful methods for affine Transformations """
     def __init__(self, transform ):
         self.geotransform=transform
         self.scaleX=self.geotransform[1]
@@ -82,7 +88,7 @@ class DataSetCache:
             return self.dict[name]
         result = gdal.Open(name)
         if result is None:
-            print("Error openenig:",NameError)
+            print("Error opening: %s" % NameError)
             sys.exit(1)
         if len(self.queue)==self.cacheSize:
             toRemove = self.queue.pop(0)
@@ -103,10 +109,10 @@ class tile_info:
     def __init__(self,xsize,ysize,tileWidth,tileHeight):
         self.tileWidth=tileWidth
         self.tileHeight=tileHeight
-        self.countTilesX= xsize / tileWidth
-        self.countTilesY= ysize / tileHeight
-        self.lastTileWidth = xsize - self.countTilesX *  tileWidth
-        self.lastTileHeight = ysize - self.countTilesY *  tileHeight
+        self.countTilesX= int(xsize / tileWidth)
+        self.countTilesY= int(ysize / tileHeight)
+        self.lastTileWidth = int(xsize - self.countTilesX *  tileWidth)
+        self.lastTileHeight = int(ysize - self.countTilesY *  tileHeight)
 
         if (self.lastTileWidth > 0 ):
             self.countTilesX=self.countTilesX+1
@@ -164,6 +170,9 @@ class mosaic_info:
            self.ct = ct.Clone()
         else:
            self.ct = None
+        self.ci = [0] * self.bands
+        for iband in range(self.bands):
+            self.ci[iband] = fhInputTile.GetRasterBand(iband + 1).GetRasterColorInterpretation()
 
         extent = self.ogrTileIndexDS.GetLayer().GetExtent()
         self.ulx = extent[0];
@@ -200,7 +209,7 @@ class mosaic_info:
         if envelope is None:
             return None
 
-        #enlarge to query rect if necessairy
+        #enlarge to query rect if necessary
         envelope= ( min(minx,envelope[0]),max(maxx,envelope[1]),
                     min(miny,envelope[2]),max(maxy,envelope[3]))
 
@@ -221,34 +230,53 @@ class mosaic_info:
             featureName =  feature.GetField(0)
             sourceDS=self.cache.get(featureName)
             dec = AffineTransformDecorator(sourceDS.GetGeoTransform())
-            #calculate read and write offsets
-            readOffsetX =int(round((minx-dec.ulx) / self.scaleX))
-            readOffsetY =int(round((maxy-dec.uly) / self.scaleY))
-            writeOffsetX=0
-            if readOffsetX<0:
-                writeOffsetX=readOffsetX*-1;
-                readOffsetX=0
-            writeOffsetY=0
-            if readOffsetY<0:
-                writeOffsetY=readOffsetY*-1;
-                readOffsetY=0
-            #calculate read and write dimensions
-            readX=min(resultSizeX,sourceDS.RasterXSize-readOffsetX,resultSizeX-writeOffsetX)
-            if readX<=0:
-                continue
-            readY=min(resultSizeY,sourceDS.RasterYSize-readOffsetY,resultSizeY-writeOffsetY)
-            if readY<=0:
+
+            dec.lrx = dec.ulx + sourceDS.RasterXSize * dec.scaleX
+            dec.lry = dec.uly + sourceDS.RasterYSize * dec.scaleY
+
+            # Find the intersection region
+            tgw_ulx = max(dec.ulx, minx)
+            tgw_lrx = min(dec.lrx, maxx)
+            if self.scaleY < 0:
+                tgw_uly = min(dec.uly, maxy)
+                tgw_lry = max(dec.lry, miny)
+            else:
+                tgw_uly = max(dec.uly, maxy)
+                tgw_lry = min(dec.lry, miny)
+
+            # Compute source window in pixel coordinates.
+            sw_xoff = int((tgw_ulx - dec.ulx) / dec.scaleX)
+            sw_yoff = int((tgw_uly - dec.uly) / dec.scaleY)
+            sw_xsize = int((tgw_lrx - dec.ulx) / dec.scaleX + 0.5) - sw_xoff
+            sw_ysize = int((tgw_lry - dec.uly) / dec.scaleY + 0.5) - sw_yoff
+            if sw_xsize <= 0 or sw_ysize <= 0:
                 continue
 
-#            print "READ",readOffsetX,readOffsetY,readX,readY
+            # Compute target window in pixel coordinates
+            tw_xoff = int((tgw_ulx - minx) / self.scaleX)
+            tw_yoff = int((tgw_uly - maxy) / self.scaleY)
+            tw_xsize = int((tgw_lrx - minx) / self.scaleX + 0.5) - tw_xoff
+            tw_ysize = int((tgw_lry - maxy) / self.scaleY + 0.5) - tw_yoff
+            if tw_xsize <= 0 or tw_ysize <= 0:
+                continue
 
-            for bandNr in range(1,self.bands+1):
+            assert tw_xoff >= 0
+            assert tw_yoff >= 0
+            assert sw_xoff >= 0
+            assert sw_yoff >= 0
+            
+            for bandNr in range(1, self.bands + 1):
                 s_band = sourceDS.GetRasterBand( bandNr )
                 t_band = resultDS.GetRasterBand( bandNr )
                 if self.ct is not None:
                     t_band.SetRasterColorTable(self.ct)
-                data = s_band.ReadRaster( readOffsetX,readOffsetY,readX,readY, readX,readY, self.band_type )
-                t_band.WriteRaster(writeOffsetX,writeOffsetY,readX,readY,data )
+                t_band.SetRasterColorInterpretation(self.ci[bandNr-1])
+                
+                data = s_band.ReadRaster( sw_xoff, sw_yoff, sw_xsize, sw_ysize, tw_xsize, tw_ysize, self.band_type )
+                if data is None:                    
+                    print(gdal.GetLastErrorMsg())
+
+                t_band.WriteRaster(tw_xoff, tw_yoff, tw_xsize, tw_ysize, data )
 
         return resultDS
 
@@ -270,7 +298,11 @@ class mosaic_info:
 def getTileIndexFromFiles( inputTiles, driverTyp):
 
     if Verbose:
-        print("Building internal Index for %d tile(s) ..." % len(inputTiles), end=' ')
+        from sys import version_info
+        if version_info >= (3,0,0):
+            exec('print("Building internal Index for %d tile(s) ..." % len(inputTiles), end=" ")')
+        else:
+            exec('print "Building internal Index for %d tile(s) ..." % len(inputTiles), ')
 
     ogrTileIndexDS = createTileIndex("TileIndex",TileIndexFieldName,None,driverTyp);
     for inputTile in inputTiles:
@@ -311,13 +343,19 @@ def tileImage(minfo, ti ):
 
     """
 
-
+    global LastRowIndx
+    LastRowIndx=-1
     OGRDS=createTileIndex("TileResult_0", TileIndexFieldName, Source_SRS,TileIndexDriverTyp)
 
 
     yRange = list(range(1,ti.countTilesY+1))
     xRange = list(range(1,ti.countTilesX+1))
 
+    if not Quiet and not Verbose:
+        progress(0.0)
+        processed = 0
+        total = len(xRange) * len(yRange)
+
     for yIndex in yRange:
         for xIndex in xRange:
             offsetY=(yIndex-1)* ti.tileHeight
@@ -331,16 +369,28 @@ def tileImage(minfo, ti ):
                 width=ti.lastTileWidth
             else:
                 width=ti.tileWidth
-            tilename=getTileName(minfo,ti, xIndex, yIndex)
+            if UseDirForEachRow :
+                tilename=getTileName(minfo,ti, xIndex, yIndex,0)
+            else:
+                tilename=getTileName(minfo,ti, xIndex, yIndex)
             createTile(minfo, offsetX, offsetY, width, height,tilename,OGRDS)
 
+            if not Quiet and not Verbose:
+                processed += 1
+                progress(processed / float(total))
 
     if TileIndexName is not None:
-        shapeName=getTargetDir()+TileIndexName
+        if UseDirForEachRow and PyramidOnly == False:
+            shapeName=getTargetDir(0)+TileIndexName
+        else:
+            shapeName=getTargetDir()+TileIndexName
         copyTileIndexToDisk(OGRDS,shapeName)
 
     if CsvFileName is not None:
-        csvName=getTargetDir()+CsvFileName
+        if UseDirForEachRow and PyramidOnly == False:
+            csvName=getTargetDir(0)+CsvFileName
+        else:
+            csvName=getTargetDir()+CsvFileName
         copyTileIndexToCSV(OGRDS,csvName)
 
 
@@ -355,6 +405,9 @@ def copyTileIndexToDisk(OGRDS, fileName):
           break
       newFeature = feature.Clone()
       basename = os.path.basename(feature.GetField(0))
+      if UseDirForEachRow :
+          t = os.path.split(os.path.dirname(feature.GetField(0)))
+          basename = t[1]+"/"+basename
       newFeature.SetField(0,basename)
       SHAPEDS.GetLayer().CreateFeature(newFeature)
     closeTileIndex(SHAPEDS)
@@ -367,6 +420,9 @@ def copyTileIndexToCSV(OGRDS, fileName):
       if feature is None:
           break
       basename = os.path.basename(feature.GetField(0))
+      if UseDirForEachRow :
+          t = os.path.split(os.path.dirname(feature.GetField(0)))
+          basename = t[1]+"/"+basename
       csvfile.write(basename);
       geom = feature.GetGeometryRef()
       coords = geom.GetEnvelope();
@@ -414,7 +470,7 @@ def createPyramidTile(levelMosaicInfo, offsetX, offsetY, width, height,tileName,
     if MemDriver is None:
         t_fh = Driver.Create( tileName, width, height, bands,bt,CreateOptions)
     else:
-        t_fh = MemDriver.Create( tileName, width, height, bands,bt,CreateOptions)
+        t_fh = MemDriver.Create( tileName, width, height, bands,bt)
 
     if t_fh is None:
         print('Creation failed, terminating gdal_tile.')
@@ -423,10 +479,11 @@ def createPyramidTile(levelMosaicInfo, offsetX, offsetY, width, height,tileName,
 
     t_fh.SetGeoTransform( geotransform )
     t_fh.SetProjection( levelMosaicInfo.projection)
-    if levelMosaicInfo.ct is not None:
-        for band in range(1,bands+1):
-            t_band = t_fh.GetRasterBand( band )
+    for band in range(1,bands+1):
+        t_band = t_fh.GetRasterBand( band )
+        if levelMosaicInfo.ct is not None:
             t_band.SetRasterColorTable(levelMosaicInfo.ct)
+        t_band.SetRasterColorInterpretation(levelMosaicInfo.ci[band-1])
 
     res = gdal.ReprojectImage(s_fh,t_fh,None,None,ResamplingMethod)
     if  res!=0:
@@ -438,6 +495,7 @@ def createPyramidTile(levelMosaicInfo, offsetX, offsetY, width, height,tileName,
 
     if MemDriver is not None:
         tt_fh = Driver.CreateCopy( tileName, t_fh, 0, CreateOptions )
+        tt_fh.FlushCache()
 
     if Verbose:
         print(tileName + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height))
@@ -484,7 +542,7 @@ def createTile( minfo, offsetX,offsetY,width,height, tilename,OGRDS):
     if MemDriver is None:
         t_fh = Driver.Create( tilename, width, height, bands,bt,CreateOptions)
     else:
-        t_fh = MemDriver.Create( tilename, width, height, bands,bt,CreateOptions)
+        t_fh = MemDriver.Create( tilename, width, height, bands,bt)
 
     if t_fh is None:
         print('Creation failed, terminating gdal_tile.')
@@ -510,6 +568,7 @@ def createTile( minfo, offsetX,offsetY,width,height, tilename,OGRDS):
 
     if MemDriver is not None:
         tt_fh = Driver.CreateCopy( tilename, t_fh, 0, CreateOptions )
+        tt_fh.FlushCache()
 
     if Verbose:
         print(tilename + " : " + str(offsetX)+"|"+str(offsetY)+"-->"+str(width)+"-"+str(height))
@@ -579,8 +638,10 @@ def closeTileIndex(OGRDataSource):
 
 def buildPyramid(minfo,createdTileIndexDS,tileWidth, tileHeight):
 
+    global LastRowIndx
     inputDS=createdTileIndexDS
     for level in range(1,Levels+1):
+        LastRowIndx = -1
         levelMosaicInfo = mosaic_info(minfo.filename,inputDS)
         levelOutputTileInfo = tile_info(levelMosaicInfo.xsize/2,levelMosaicInfo.ysize/2,tileWidth,tileHeight)
         inputDS=buildPyramidLevel(levelMosaicInfo,levelOutputTileInfo,level)
@@ -624,6 +685,8 @@ def getTileName(minfo,ti,xIndex,yIndex,level = -1):
     """
     creates the tile file name
     """
+    global LastRowIndx
+
     max = ti.countTilesX
     if (ti.countTilesY > max):
         max=ti.countTilesY
@@ -632,10 +695,20 @@ def getTileName(minfo,ti,xIndex,yIndex,level = -1):
     if parts[0][0]=="@" : #remove possible leading "@"
        parts = ( parts[0][1:len(parts[0])], parts[1])
 
-    if Extension is None:
-        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"+parts[1]
+    if UseDirForEachRow :
+        format=getTargetDir(level)+str(yIndex)+os.sep+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"
+        #See if there was a switch in the row, if so then create new dir for row.
+        if LastRowIndx < yIndex :
+            LastRowIndx = yIndex
+            if (os.path.exists(getTargetDir(level)+str(yIndex)) == False) :
+                os.mkdir(getTargetDir(level)+str(yIndex))
     else:
-        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"+"."+Extension
+        format=getTargetDir(level)+parts[0]+"_%0"+str(countDigits)+"i"+"_%0"+str(countDigits)+"i"
+    #Check for the extension that should be used.
+    if Extension is None:
+        format=format+parts[1]
+    else:
+        format=format+"."+Extension
     return format % (yIndex,xIndex)
 
 def UsageFormat():
@@ -648,7 +721,7 @@ def UsageFormat():
 # =============================================================================
 def Usage():
      print('Usage: gdal_retile.py ')
-     print('        [-v] [-co NAME=VALUE]* [-of out_format]')
+     print('        [-v] [-q] [-co NAME=VALUE]* [-of out_format]')
      print('        [-ps pixelWidth pixelHeight]')
      print('        [-ot  {Byte/Int16/UInt16/UInt32/Int32/Float32/Float64/')
      print('               CInt16/CInt32/CFloat32/CFloat64}]')
@@ -656,6 +729,7 @@ def Usage():
      print('        [ -csv fileName [-csvDelim delimiter]]')
      print('        [-s_srs srs_def]  [-pyramidOnly] -levels numberoflevels')
      print('        [-r {near/bilinear/cubic/cubicspline/lanczos}]')
+     print('        [-useDirForEachRow]')
      print('        -targetDir TileDirectory input_files')
 
 # =============================================================================
@@ -666,9 +740,10 @@ def Usage():
 # Program mainline.
 #
 
-def main(args):
+def main(args = None):
 
     global Verbose
+    global Quiet
     global CreateOptions
     global Names
     global TileWidth
@@ -689,9 +764,12 @@ def main(args):
     global ResamplingMethod
     global Levels
     global PyramidOnly
+    global UseDirForEachRow
 
     gdal.AllRegister()
 
+    if args is None:
+        args = sys.argv
     argv = gdal.GeneralCmdLineProcessor( args )
     if argv is None:
         return 1
@@ -708,7 +786,7 @@ def main(args):
             i+=1
             BandType = gdal.GetDataTypeByName( argv[i] )
             if BandType == gdal.GDT_Unknown:
-                print('Unknown GDAL data type: ', argv[i])
+                print('Unknown GDAL data type: %s' % argv[i])
                 return 1
         elif arg == '-co':
             i+=1
@@ -717,6 +795,8 @@ def main(args):
 
         elif arg == '-v':
             Verbose = True
+        elif arg == '-q':
+            Quiet = True
 
         elif arg == '-targetDir':
             i+=1
@@ -738,23 +818,23 @@ def main(args):
             i+=1
             ResamplingMethodString=argv[i]
             if ResamplingMethodString=="near":
-                ResamplingMethod=GRA_NearestNeighbour
+                ResamplingMethod=gdal.GRA_NearestNeighbour
             elif ResamplingMethodString=="bilinear":
-                 ResamplingMethod=GRA_Bilinear
+                 ResamplingMethod=gdal.GRA_Bilinear
             elif ResamplingMethodString=="cubic":
-                 ResamplingMethod=GRA_Cubic
+                 ResamplingMethod=gdal.GRA_Cubic
             elif ResamplingMethodString=="cubicspline":
-                 ResamplingMethod=GRA_CubicSpline
+                 ResamplingMethod=gdal.GRA_CubicSpline
             elif ResamplingMethodString=="lanczos":
-                ResamplingMethod=GRA_Lanczos
+                ResamplingMethod=gdal.GRA_Lanczos
             else:
-                print("Unknown resampling method:" ,ResamplingMethodString)
+                print("Unknown resampling method: %s" % ResamplingMethodString)
                 return 1
         elif arg == '-levels':
             i+=1
             Levels=int(argv[i])
             if Levels<1:
-                print("Invalid number of levels", Levels)
+                print("Invalid number of levels : %d" % Levels)
                 return 1
         elif arg == '-s_srs':
             i+=1
@@ -784,8 +864,10 @@ def main(args):
         elif arg == '-csvDelim':
             i+=1
             CsvDelimiter=argv[i]
+        elif arg == '-useDirForEachRow':
+            UseDirForEachRow=True
         elif arg[:1] == '-':
-            print('Unrecognised command option: ', arg)
+            print('Unrecognised command option: %s' % arg)
             Usage()
             return 1
 
@@ -807,17 +889,24 @@ def main(args):
         Usage()
         return 1
 
+    # create level 0 directory if needed
+    if(UseDirForEachRow and PyramidOnly==False) :
+        leveldir=TargetDir+str(0)+os.sep
+        if (os.path.exists(leveldir)==False):
+            os.mkdir(leveldir)
+
     if Levels > 0:    #prepare Dirs for pyramid
-        for levelIndx in range (1,Levels+1):
+        startIndx=1
+        for levelIndx in range (startIndx,Levels+1):
             leveldir=TargetDir+str(levelIndx)+os.sep
             if (os.path.exists(leveldir)):
                 continue
             os.mkdir(leveldir)
             if (os.path.exists(leveldir)==False):
-                print("Cannot create level dir:", leveldir)
+                print("Cannot create level dir: %s" % leveldir)
                 return 1
             if Verbose :
-                print("Created level dir: ",leveldir)
+                print("Created level dir: %s" % leveldir)
 
 
     Driver = gdal.GetDriverByName(Format)
@@ -830,7 +919,7 @@ def main(args):
 
 
     DriverMD = Driver.GetMetadata()
-    Extension=DriverMD.get(DMD_EXTENSION);
+    Extension=DriverMD.get(gdal.DMD_EXTENSION);
     if 'DCAP_CREATE' not in DriverMD:
         MemDriver=gdal.GetDriverByName("MEM")
 
@@ -888,6 +977,9 @@ def initGlobals():
     global ResamplingMethod
     global Levels
     global PyramidOnly
+    global LastRowIndx
+    global UseDirForEachRow
+
 
     Verbose=False
     CreateOptions = []
@@ -907,13 +999,17 @@ def initGlobals():
 
     Source_SRS=None
     TargetDir=None
-    ResamplingMethod=GRA_NearestNeighbour
+    ResamplingMethod=gdal.GRA_NearestNeighbour
     Levels=0
     PyramidOnly=False
+    LastRowIndx=-1
+    UseDirForEachRow=False
+
 
 
 #global vars
 Verbose=False
+Quiet=False
 CreateOptions = []
 Names=[]
 TileWidth=256
@@ -930,12 +1026,12 @@ CsvDelimiter=";"
 CsvFileName=None
 Source_SRS=None
 TargetDir=None
-ResamplingMethod=GRA_NearestNeighbour
+ResamplingMethod=gdal.GRA_NearestNeighbour
 Levels=0
 PyramidOnly=False
-
+LastRowIndx=-1
+UseDirForEachRow=False
 
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))
-
